package handlers

import (
	"fmt"

	"github.com/go-openapi/runtime/middleware"
	"github.com/jmoiron/sqlx"
	"github.com/rs/zerolog/hlog"
	"orus.io/orus-io/go-orusapi/database"

	redner "orus.io/orus-io/rednerd/lib"
	"orus.io/orus-io/rednerd/models"
	"orus.io/orus-io/rednerd/restapi/operations/template_management"
)

func NewAccountTemplatePreviewHandler(db *sqlx.DB, r *redner.Renderer) *AccountTemplatePreview {
	return &AccountTemplatePreview{db, r}
}

type AccountTemplatePreview struct {
	db       *sqlx.DB
	renderer *redner.Renderer
}

func (h *AccountTemplatePreview) Handle(
	params template_management.AccountTemplatePreviewParams,
	principal *models.Principal,
) middleware.Responder {
	ctx := params.HTTPRequest.Context()
	log := hlog.FromRequest(params.HTTPRequest)
	db := database.NewSQLHelper(ctx, h.db, *log)

	template, err := redner.LoadTemplate(&db, params.Account, params.Name)
	if err != nil {
		return InternalError(err, log, false)
	}

	etag := template.ETag
	if params.HTTPRequest.Header.Get("If-None-Match") == etag {
		return template_management.NewAccountTemplatePreviewNotModified().WithETag(etag)
	}

	mimetype := "image/png"

	if params.Format == "html" {
		mimetype = "text/html"
	}

	documents, err := h.renderer.Render(
		ctx,
		principal.Subject,
		nil, nil,
		&models.Document{
			Type:       template.Produces,
			Body:       template.Body,
			BodyFormat: template.BodyFormat,
		},
		models.Metadata{},
		mimetype,
		false,
	)
	if err != nil {
		return InternalError(fmt.Errorf("could not render document: %w", err), log, false)
	}

	if len(documents) != 1 {
		return InternalError(fmt.Errorf("expected 1 rendered document, got %d", len(documents)), log, false)
	}

	doc := documents[0]

	return NewDocumentFileResponse().
		WithETag(etag).
		WithDocument(doc)
}
